from webob import Request, Response
from .Route import Router
from .utils import default_response
#from ....app.app import run
class API:
    

    def __init__(self):
        router = Router()
        #self.collect_routes()
        self.routes = {}
        self.routes = router.get_routes()
        self.routes["/testss"] = self.collect_routes
    
    # main application call
    def __call__(self,environ,start_response):
        req = Request(environ)

        res = self.handle_requests(req)
        #res = Response()
        #res.content_type = "text/html"
        #res.text = "Hello, World" + "\n" + "<h1 style='background-color:red;color:white;'>MANLOW CHARUMBIRA </h1>"
        #print(f"=======request object path======{req.path}, method====={req.method}")
        print(f"=======routes======{self.routes}")
        #return res(environ,start_response)
    # collect website routes, import the app module and run the Run function in it
    def collect_routes(self,req,res):
        res.text = "hello"
    def handle_requests(self,request):
        found = False
        res = Response()
        while len(self.routes) >0:
            for path,handler in self.routes.items():
                if path == request.path:
                    print(f"===match found====path--{request.path}, handler==={handler},routes=={self.routes}")
                    handler(request,res)
                    #res.text = handler
                    found = True
                    return res
        if not found:
            default_response(res)
        
app = API()


class BodyContent:
    def __init__(self,files=[]):
       pass
    
    def body(self,content="Hello World"):
        open_tag = "\n" + "<body>" + "\n"
        open_tag = open_tag + content
        return open_tag


from .engine.views.all import SiteWrapper
from .engine.routing.Route import Router
from .engine.routing.index import app
from .engine.routing.waitress_server import run_server
from webob import Response
resp = Response()

st = SiteWrapper()
ec = st.web_document
route = Router()
def view(req,res):
    res.text = "hello world test"
#route.route("/test",st.web_document)
route.route("/test",view)
#app = API()
run_server(app)



class Router:
    routes = {}
    def __init__(self):
        pass
    
    def route(self,path,component):
        # to be updated for class and functional components
        self.routes[path] = component



from .allWidgets.headContent import HeadContent
from .allWidgets.body import BodyContent
from .allWidgets.footer import Footer

class SiteWrapper:
    def __init__(self):
        self.head_ = ""
        self.body_ = ""
        self.footer_ = ""

    
    def head(self,title="Document",files=[]):
        head = HeadContent(files=files)
        self.head_ = head.head(title)
    
    def body(self,files=[],content="Hello World"):
        body = BodyContent(files=files)
        self.body_ = body.body(content=content)
    def footer(self,content="Brought  To You With Love from HotWeb",files=[]):
        footer = Footer()
        self.footer_ = footer.footer(content=content,files=files)
    
    def web_document(self,req,res):
        if len(self.head_) ==0:
            self.head()
        if len(self.body_) ==0:
            self.body()
        if len(self.footer_) ==0:
            self.footer()
        
        doc = self.head_ + "\n" + self.body_ + "\n" + self.footer_
        res.text = doc
        print(doc)
        #return doc




import os
from waitress import serve
#from .index import app
this_dir = os.path.dirname(os.path.abspath(__file__))
os.chdir(this_dir)

# run the server
def run_server(app_):
    serve(app_,host="127.0.0.1",port=8001,url_prefix="")



from setuptools import setup, find_packages


setup(
    name="hotweb",
    version="0.7.11",
    license="MIT",
    author = "Real Manlow,aka ManlowCharumbira",
    author_email="realmanlow20@gmail.com",
    #packages=["hotweb"],
    package_dir = {'':'hotweb'},
    keywords = "hotweb python-web-framework python web framework fast light secure",
    install_requires = [
        "parse","waitress","webob","requests",
    ]
)



class Router:
    routes = {}
    def __init__(self):
        pass
    def route(self,path,component):
        #for path_ in self.routes.keys():
            #if path_ == path:
                #print("===========path taken========")
                #return
        # component is the view of the given page path
        self.routes[path] = component
    
    def get_routes(self):
        return self.routes
    
    #def example_handler(self,req,res,**args,**kwargs):
        #res.text = "Entry or Home Handler executed"

#app =Router()
#app.route("/home","this is home page")
#app.route("/aboute","this is home page")
#app.route("/services","this is home page")
#print(app.get_routes())


import os
componets = ["Slider","DARKMODE","Nav","Button","icons","Input","Link","FAB","RATING","TOGGLE","FORM","SWITCH","AVATAR","LIST","TABLE","DIVIDER","ALERT","SNACKBAR","TYPOGRAPHY","BADGE","TOOLTIP","MODAL","PROGRESS","SKELETON","CARD","ACCORDION","DRAWER","BOTTOMNAVIGARION","PAPER","BREADCRUMPS","TABS","BOX","STEPPER","MENU","PAGINATION","SPEEDDIAL","IMAGE","VIDEO","ICON"]
for item in componets:
    os.mkdir(item.lower().capitalize())
    #os.rmdir(item)


from webob import Response,Request
import time
from routes import Router
def eg(req,res):
    found = f"found 1"
    count = "0"
    for k in range(1,10):
        if k%10 ==0:
            count = count + str(int(count)+1)
            found = found + "<script>console.log('nor-manlow')</script> k==0 \n" + str(count)
            res.text = found 
            #time.sleep(5)
        else:
            #f"test :k=={k}: modulo not zero=========" + window.location='nor-manlow' window.location='manlow' document.title = 'the doc' 
            count = count + str(int(count)+1)
            res.text =  "<body><script>console.log('manlow');document.title = 'the doc'</script> k <>==0</body> \n"+ str(count)
            #time.sleep(10)
class API:
    def __init__(self):
        self.routes = {"/":eg}
    
    def __call__(self,environ,start_response):
        req = Request(environ)
        res = self.handle_request(req)
        print(f"res====={res}")
        #res.text = "hello"
        return res(environ,start_response)
    
    
    def handle_request(self,req):
        res = Response()
        handler,handler_ = self.find_handler(req_path=req.path)
        if handler_:
            handler(req,res)
        else:
            res.text = "File not foundr"
        return res
    def find_handler(self,req_path):
        while len(self.routes.keys()) >0:
            for path,handler in self.routes.items():
                if path == req_path:
                    return [handler,True]
            break
        return ["",False]

app = API()